
# proveravanje radnog direktorijuma
getwd()


# ucitavanje potrebnih biblioteka
library(tidyverse)
library(ggplot2)
library(dplyr)
library(RColorBrewer)
library(psych)
library(gridExtra)
library(mice)
library(reshape2)
library(RColorBrewer)
library(class)
library(caret)
library(xgboost)
library(ggrepel)




# ucitavanje i pregledavanje skupa podataka
health_dataset = read_csv("heart_2020_cleaned.csv")
view(health_dataset)


# pregledanje prvih redova skupa
head(health_dataset)
glimpse(health_dataset)


# pregled dimenzija skupa podataka
dim(health_dataset)


# pregledanje svih naziva i tipova varijabli skupa
str(health_dataset)


# raspon vrednosti u svakoj od kolona
sapply(health_dataset, range)


# pregled osnovnih infomracija o varijablama skupa
summary(health_dataset)


# vizuelizacija distribucije podataka kolone HeartDisease
ggplot(health_dataset)+
  geom_bar(aes(x = HeartDisease, fill = HeartDisease, color = HeartDisease)) +
  labs(x = "HeartDisease", y = "Count") +
  scale_fill_manual(values = c("skyblue","pink"))+
  scale_color_manual(values = c("blue", "red"))


# brojcana distribucija podataka kolone HeartDisease
class_count = table(health_dataset$HeartDisease)
class_count


# procentualna distribucija podataka kolone HeartDisease
proportions = health_dataset %>%
  group_by(HeartDisease) %>%
  summarise(count = n()) %>%
  mutate(percentage = count / sum(count) * 100)
proportions


# pregled broja istih redova unutar skupa podataka
duplicate = duplicated(health_dataset)
sum(duplicate)
glimpse(health_dataset[duplicate>0,])




############################################################################################################################################
#     ANALIZA VARIJABLI SKUPA PODATAKA
############################################################################################################################################


###################################################
# Analiza numeričkih varijabli

# pregled numeričkih varijabli
numeric_data = health_dataset %>% select_if(is.numeric)
colnames(numeric_data)

 
 
 # funkcija za izvlačenje statistike
 plot_boxplot_with_stats = function(dataset, column_name) {
   # Izvlačenje statistika za datu kolonu
   stats <- summary(dataset[[column_name]])
   median_val <- stats[["Median"]]
   q1_val <- stats[["1st Qu."]]
   q3_val <- stats[["3rd Qu."]]
   
   # Kreiranje boxplota za datu kolonu
   ggplot(dataset, aes(x = 1, y = .data[[column_name]])) +
     geom_boxplot(fill = "skyblue", color = "darkblue") +
     
     # Dodavanje oznaka sa različitim y koordinatama
     annotate("label", x = 0.8, y = q3_val, label = paste("Q3:", round(q3_val, 2)), 
              hjust = 0, color = "black", fill = "white", size = 4) +
     annotate("label", x = 1.15, y = median_val, label = paste("Med:", round(median_val, 2)), 
              hjust = 0, color = "black", fill = "white", size = 4) +
     annotate("label", x = 0.8, y = q1_val, label = paste("Q1:", round(q1_val, 2)), 
              hjust = 0, color = "black", fill = "white", size = 4) +
     labs(title = paste("Boxplot of", column_name), y = paste(column_name, "values"), x = "")
 }
 
plot_histogram = function(dataset, column_name, binwidth = 5) {
   
   # Kreiranje histograma za datu kolonu
   ggplot(dataset, aes(x = .data[[column_name]])) +       
     geom_histogram(binwidth = binwidth,     
                    fill = "lightgreen",    
                    color = "darkgreen") +   
     labs(title = paste("Histogram of", column_name), 
          x = column_name,                  
          y = "Frequency")
 }
 
 
####################
# BMI
describe(health_dataset$BMI)
 
# boxplot
plot_boxplot_with_stats(health_dataset, "BMI")
 
# broj ispitanika sa normalnom vrednošću BMI
normal_BMI_count = sum(health_dataset$BMI >= 18.5 & 
                         health_dataset$BMI <= 25)
normal_BMI_perc = round((normal_BMI_count*100)/nrow(health_dataset),2)
normal_BMI_perc

# broj autlajera
boxplot_stats = boxplot.stats(health_dataset$BMI)
outliers = boxplot_stats$out
length(outliers)

# raspodela violin
ggplot(health_dataset, aes(x = factor(1), y = BMI)) +
  geom_violin(fill = "pink", color = "violetred", alpha = 0.5) +
  labs(x = "", y = "BMI") + 
  coord_flip()

# hist
plot_histogram(health_dataset, "BMI", 5)




####################
# PhysicalHealth
describe(health_dataset$PhysicalHealth)
 
plot_boxplot_with_stats(health_dataset, "PhysicalHealth")

ggplot(health_dataset, aes(x = factor(1), y = PhysicalHealth)) +
  geom_violin(fill = "pink", color = "violetred", alpha = 0.5) +
  labs(x = "", y = "PhysicalHealth") + 
  coord_flip()

plot_histogram(health_dataset, "PhysicalHealth", 5)


####################
#MentalHealth
describe(health_dataset$MentalHealth)

plot_boxplot_with_stats(health_dataset, "MentalHealth")

plot_histogram(health_dataset, "MentalHealth", 5)


ggplot(health_dataset, aes(x = factor(1), y = MentalHealth)) +
  geom_violin(fill = "pink", color = "violetred", alpha = 0.5) +
  labs(x = "", y = "MentalHealth") + 
  coord_flip()

plot_histogram(health_dataset, "MentalHealth", 5)


####################
#SleepTime
describe(health_dataset$SleepTime)

plot_boxplot_with_stats(health_dataset, "SleepTime")

ggplot(health_dataset, aes(x = factor(1), y = SleepTime)) +
  geom_violin(fill = "pink", color = "violetred", alpha = 0.5) +
  labs(x = "", y = "SleepTime") + 
  coord_flip()

plot_histogram(health_dataset, "SleepTime", 1)

health_dataset$SleepTime


###################################################
# Analiza kategorijskih varijabli


categorical_data = health_dataset %>% select_if(~ is.character(.))
colnames(categorical_data)


str(health_dataset)

# definisanje funkcije za crtanje barcharta
custom_barplot = function(dataset, column_name) {
  ggplot(dataset) +
    geom_bar(aes(x = .data[[column_name]], fill = .data[[column_name]], color = .data[[column_name]])) +
    labs(x = column_name, y = "Count") +
    scale_fill_manual(values = c("paleturquoise", "peachpuff")) +
    scale_color_manual(values = c("darkcyan", "orange")) 
}


# definisanje funkcije za crtanje piecharta
library(ggrepel)

custom_pie_chart = function(dataset, column_names) {
  # Kreiranje praznog data frame-a za sve kategorije i procente
  combined_data_for_plot <- data.frame()
  
  for (column_name in column_names) {
    # Osiguravanje da je svaka kolona faktorska (kategorijska)
    dataset[[column_name]] <- as.factor(dataset[[column_name]])
    
    # Računanje procenata
    counts <- table(dataset[[column_name]])
    percentages <- round(prop.table(counts) * 100, 1)
    
    # Prebacivanje u format potreban za pie chart
    data_for_plot <- data.frame(
      Category = names(percentages),
      Percentage = as.numeric(percentages),
      Count = as.numeric(counts),
      Column = column_name  # Dodajemo ime kolone da možemo koristiti facet_wrap
    )
    
    # Kombinovanje podataka za sve kolone
    combined_data_for_plot <- rbind(combined_data_for_plot, data_for_plot)
  }
  
  # Crtanje pie chart-a sa facet_wrap
  ggplot(combined_data_for_plot, aes(x = "", y = Percentage, fill = Category)) +
    geom_bar(stat = "identity", width = 1) +  # Kreiranje bar grafikona
    coord_polar(theta = "y") +  # Konvertovanje u koordinate za pie chart
    theme_void() +  # Uklanjanje pozadine i osa
    geom_label_repel(aes(label = paste0(Percentage, "%")),  # Dodavanje oznaka sa procentima
                     position = position_stack(vjust = 0.5),  # Sprečavanje preklapanja oznaka
                     size = 4, show.legend = FALSE,  # Povećanje veličine teksta
                     box.padding = 0.5,  # Pomeranje oznaka ka spolja
                     point.padding = 0.5, # Održavanje razmaka između oznaka
                     direction = "y") +  # Kontrolisanje pravca repela (ka spolja)
    labs(title = "Pie Charts of Selected Columns") +
    scale_fill_brewer(palette = "Set3") +  # Korišćenje automatske palete boja
    theme(
      plot.title = element_text(size = 16, face = "bold"),  # Povećanje veličine glavnog naslova
      strip.text = element_text(size = 14, face = "bold")   # Povećanje veličine faceta (naslova panela)
    ) +
    facet_wrap(~Column)  # Prikazivanje po kolonama
}

                                    

####################
# Smoking

custom_barplot(health_dataset,"Smoking")
custom_pie_chart(health_dataset, "Smoking")


####################
# AlcoholDrinking
custom_barplot(health_dataset,"AlcoholDrinking")
custom_pie_chart(health_dataset, "AlcoholDrinking")


####################
# Stroke
custom_barplot(health_dataset,"Stroke")
custom_pie_chart(health_dataset, "Stroke")


####################
#DiffWalking
custom_barplot(health_dataset,"DiffWalking")
custom_pie_chart(health_dataset, "DiffWalking")

####################
# Sex
unique(health_dataset$Sex)
custom_barplot(health_dataset,"Sex")
custom_pie_chart(health_dataset, "Sex")


####################
# AgeCategory
unique(health_dataset$AgeCategory)
ggplot(health_dataset)+
  geom_bar(aes(x = AgeCategory, fill = AgeCategory)) +
  labs(x = "AgeCategory", y = "Count") +
  theme_minimal()
custom_pie_chart(health_dataset, "AgeCategory")
# procentualna raspodela



####################
# Race
unique(health_dataset$Race)
ggplot(health_dataset)+
  geom_bar(aes(x = Race, fill = Race, color = Race)) +
  labs(x = "Race", y = "Count") 

#procentualna zastupljenost
custom_pie_chart(health_dataset, "Race")


####################
#Diabetic
unique(health_dataset$Diabetic)
ggplot(health_dataset)+
  geom_bar(aes(x = Diabetic, fill = Diabetic, color = Diabetic)) +
  labs(x = "Race", y = "Count") 

# procentualna zastupljenost
custom_pie_chart(health_dataset, "Diabetic")


####################
#PhysicalActivity
unique(health_dataset$PhysicalActivity)
custom_barplot(health_dataset,"PhysicalActivity")
custom_pie_chart(health_dataset, "PhysicalActivity")


####################
# GenHealth
ggplot(health_dataset)+
  geom_bar(aes(x = GenHealth, fill = GenHealth, color = GenHealth)) +
  labs(x = "GenHealth", y = "Count") 

#procenutalna raspodela
custom_pie_chart(health_dataset, "GenHealth")


####################
#Asthma
custom_barplot(health_dataset,"Asthma")
custom_pie_chart(health_dataset, "Asthma")


####################
#KidneyDisease
custom_barplot(health_dataset,"KidneyDisease")
custom_pie_chart(health_dataset, "KidneyDisease")


####################
#SkinCancer
custom_barplot(health_dataset,"SkinCancer")
custom_pie_chart(health_dataset, "SkinCancer")



############################################################################################################################################
#   ANALIZA NETIPICNIH VREDNOSTI
############################################################################################################################################


####################
# Physical i Mental health
summary(health_dataset$MentalHealth)
summary(health_dataset$PhysicalHealth)

####################
# SleepTime
# broj ispitanika koji su naveli da prosecno spavaju 24 sata mesecno
sum(health_dataset$SleepTime == 24)

# zamena tih ispitanika NA vrednostima
health_dataset$SleepTime[health_dataset$SleepTime == 24] = NA
summary(health_dataset$SleepTime)


####################
# Sex
sum(health_dataset$Sex == "Male" & health_dataset$Diabetic == "Yes (during pregnancy)")


####################
# PhysicalHealth i MentalHealth
summary(health_dataset$MentalHealth)
summary(health_dataset$PhysicalHealth)


######################################################################
# DUPLIKATI
duplicate = duplicated(health_dataset)
sum(duplicate)

health_dataset = distinct(health_dataset)
nrow(health_dataset)


######################################################################
# FAKTORIZACIJA KATEGORIJSKIH PODATAKA
health_dataset = health_dataset %>%
  mutate(across(where(is.character), as.factor))

str(health_dataset)

######################################################################
# RAD SA NA VREDNOSTIMA 


####################
# Vestacko dodavanje

#odabir nasumičnog procenta za SleepTime kolone
set.seed(012)
sleepTimePerc = sample(1:10, 1)
sleepTimePerc
#izracunavanje tacnog broja redova
sleepTimeRowNum = round(nrow(health_dataset) * (sleepTimePerc/100))
sleepTimeRowNum


#odabir nasumičnog procenta za GenHealth kolone
set.seed(123)
GenHealthPerc = sample(1:10, 1)
GenHealthPerc
#izracunavanje tacnog broja redova
GenHealthRowNum = round(nrow(health_dataset) * (GenHealthPerc/100))
GenHealthRowNum


#odabir nasumičnog procenta Sex kolone
set.seed(234)
SexPerc = sample(1:10, 1)
SexPerc
#izracunavanje tacnog broja redova
SexRowNum = round(nrow(health_dataset) * (SexPerc/100))
SexRowNum


# kreiranje kopije seta podataka zarad uporedbe
copyOfDataset = health_dataset
health_dataset = copyOfDataset


# samo dodavanje 

#SleepTime
set.seed(345)
sample = sample.int(n = nrow(health_dataset), size = sleepTimeRowNum)
health_dataset$SleepTime[sample] = NA

#GenHealth
set.seed(456)
sample = sample.int(n = nrow(health_dataset), size = GenHealthRowNum)
health_dataset$GenHealth[sample] = NA

#Sex
set.seed(678)
sample = sample.int(n = nrow(health_dataset), size = SexRowNum)
health_dataset$Sex[sample] = NA

colSums(is.na(health_dataset))



########################################
# Popunjavanje NA vrednosti


####################
#SleepTime
 #podsetnik raspodele unutar ove kolone
summary_stats = summary(health_dataset$SleepTime)
summary_stats

median_val = unname(summary_stats["Median"])
health_dataset$SleepTime[is.na(health_dataset$SleepTime)] = median_val

sum(is.na(health_dataset$SleepTime))

#provera tacnosti
valid_rows = !is.na(health_dataset$SleepTime) & !is.na(copyOfDataset$SleepTime)
mean((copyOfDataset$SleepTime[valid_rows] - health_dataset$SleepTime[valid_rows])^2)


####################
#Sex
  # da li ima NA vrednosti kod kojih je Diabetic vrednost "Yes (during pregnancy)"
sum(is.na(health_dataset$Sex) & health_dataset$Diabetic == "Yes (during pregnancy)")
  
  # popunjavanje tih vrednosti
health_dataset$Sex[is.na(health_dataset$Sex) & 
                     health_dataset$Diabetic == "Yes (during pregnancy)"] = "Female"


females = health_dataset[health_dataset$Sex == "Female",]
summary(females$MentalHealth)


males = health_dataset[health_dataset$Sex == "Male",]
summary(males$MentalHealth)

ggplot(data = health_dataset, mapping = aes(x = MentalHealth)) +
  geom_freqpoly(mapping = aes(colour = Sex), binwidth = 5)


subDataset = health_dataset[, !(names(health_dataset) == "GenHealth")]
imputed = mice(subDataset, m = 1, method = 'logreg', seed = 789)
completedSex = complete(imputed)

health_dataset$Sex = completedSex$Sex
sum(is.na(health_dataset$Sex))

# uporedjivanje sa stvarnim vrednostima
numOfMistakes = sum(copyOfDataset$Sex != health_dataset$Sex)
numOfMistakes
# procenat u kome je pogrešeno
round((numOfMistakes/2726)*100)


####################
#GenHealth
counts = table(health_dataset$GenHealth)
percentages = round(prop.table(counts) * 100, 1)
percentages                      


# veza izmedju PhysicalHealth i GenHealth
ggplot(data = health_dataset, mapping = aes(x = PhysicalHealth)) +
  geom_freqpoly(mapping = aes(colour = GenHealth), binwidth = 5)


plot(health_dataset$GenHealth, health_dataset$PhysicalHealth,
     col = c("lightblue", "lightgreen", "lightpink", "lightgoldenrod", "purple"),
     main = "Physical Health vs General Health",
     xlab = "General Health",
     ylab = "Days of Poor Physical Health")

# dodavanje kolone koja predstavlja broj bolesti od kojih ispitanici boluju
health_dataset$TotalDiseases =  rowSums(health_dataset[,c(
  "Stroke", "DiffWalking", "Diabetic", "Asthma", "KidneyDisease", "SkinCancer")] == "Yes")


# odnos broja bolesti i generalnog zdravlja
plot(TotalDiseases ~ GenHealth, data = health_dataset,
        main = "Number of Diseases vs General Health",
        xlab = "General Health",
        ylab = "Total Number of Diseases",
        col = c("lightblue", "lightgreen", "lightpink", "lightgoldenrod", "purple"))


# imputacija 
numOfNaS = sum(is.na(health_dataset$GenHealth))
numOfNaS


# Poor
health_dataset$GenHealth[is.na(health_dataset$GenHealth) &
                  (health_dataset$PhysicalHealth > 18 | health_dataset$TotalDiseases > 2)] = "Poor"

# Fair
health_dataset$GenHealth[is.na(health_dataset$GenHealth) &
                  health_dataset$PhysicalHealth >= 5 & health_dataset$PhysicalHealth <= 18 & 
                  health_dataset$TotalDiseases >= 1 & health_dataset$TotalDiseases <= 2] = "Fair"

# Good
health_dataset$GenHealth[is.na(health_dataset$GenHealth) &
                  health_dataset$PhysicalHealth >= 1 & health_dataset$PhysicalHealth <= 5 &
                  health_dataset$TotalDiseases >= 0 & health_dataset$TotalDiseases <= 2] = "Good"


# Very good i Excellent 
na_indices = which(is.na(health_dataset$GenHealth))
set.seed(123)  
shuffled_indices = sample(na_indices)

# Izračunaj broj elemenata za 80% i 20%
n = length(shuffled_indices)
n_very_good = round(0.8 * n)
n_excellent = n - n_very_good

# Dodeli prva 80% nasumičnih vrednosti kao "Very good"
health_dataset$GenHealth[shuffled_indices[1:n_very_good]] = "Very good"

# Dodeli preostalih 20% nasumičnih vrednosti kao "Excellent"
health_dataset$GenHealth[shuffled_indices[(n_very_good + 1):n]] = "Excellent"

# provera je li sve lepo zamenjeno
sum(is.na(health_dataset$GenHealth))


numOfMistakes = sum(copyOfDataset$GenHealth != health_dataset$GenHealth)
numOfMistakes

# procenat u kome je pogrešeno
(numOfMistakes/numOfNaS)*100





############################################################################################################################################
#   EDA
############################################################################################################################################


#######################################################################
# NUMERICKE
numericalVars = health_dataset %>% 
  select_if(is.numeric)
correlation = round(cor(numericalVars), 2)

# Prebacivanje korelacionog matriksa u dugi format
melted_corr = melt(correlation)

# Kreiranje heatmapa sa ggplot2
ggplot(data = melted_corr, aes(Var1, Var2, fill = value)) +
  geom_tile(color = "white") +
  scale_fill_gradientn(colors = brewer.pal(n = 9, name = "YlOrBr"), limits = c(-1, 1)) +
  geom_text(aes(label = value), color = "black", size = 4) +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 45, hjust = 1)) +
  labs(fill = "Correlation", x = "", y = "")



# BMI
ggplot(data = health_dataset, 
       mapping = aes(x = HeartDisease, y = BMI, fill = HeartDisease)) +
  geom_boxplot() 
  

# PhysicalHealth
ggplot(data = health_dataset, 
       mapping = aes(x = HeartDisease, y = PhysicalHealth, fill = HeartDisease)) +
  geom_boxplot() 


# MentalHealth
ggplot(data = health_dataset, 
       mapping = aes(x = HeartDisease, y = MentalHealth, fill = HeartDisease)) +
  geom_boxplot() 


# SleepTime
ggplot(data = health_dataset, 
       mapping = aes(x = HeartDisease, y = SleepTime, fill = HeartDisease)) +
  geom_boxplot() 


# TotalDisease
ggplot(data = health_dataset, 
       mapping = aes(x = HeartDisease, y = TotalDiseases, fill = HeartDisease)) +
  geom_boxplot() 





#######################################################################
# KATEGORIJSKE 

# funkcija za barplot 


plot_boxplot_with_stats = function(dataset, column_name) {
  # Izvlačenje statistika za datu kolonu
  stats <- summary(dataset[[column_name]])
  median_val <- stats[["Median"]]
  q1_val <- stats[["1st Qu."]]
  q3_val <- stats[["3rd Qu."]]
  
  # Kreiranje boxplota za datu kolonu
  ggplot(dataset, aes(x = 1, y = .data[[column_name]])) +
    geom_boxplot(fill = "skyblue", color = "darkblue") +
    
    # Dodavanje oznaka sa različitim y koordinatama
    annotate("label", x = 0.8, y = q3_val, label = paste("Q3:", round(q3_val, 2)), 
             hjust = 0, color = "black", fill = "white", size = 4) +
    annotate("label", x = 1.15, y = median_val, label = paste("Med:", round(median_val, 2)), 
             hjust = 0, color = "black", fill = "white", size = 4) +
    annotate("label", x = 0.8, y = q1_val, label = paste("Q1:", round(q1_val, 2)), 
             hjust = 0, color = "black", fill = "white", size = 4) +
    labs(title = paste("Boxplot of", column_name), y = paste(column_name, "values"), x = "")
}

health_dataset



create_bivariant_bar = function(data, x_col, fill_col) {
  ggplot(data, aes_string(x = x_col, fill = fill_col)) + 
    geom_bar(position = "fill") +  # Change position to "fill" for proportions
    ylab(paste0("Proportion of ", fill_col)) + 
    xlab(x_col) + 
    scale_x_discrete(labels = c("No", "Yes")) +
    scale_fill_manual(values = c("skyblue", "pink")) + 
    scale_y_continuous(labels = scales::percent)  # Display y-axis as percentages
}


# Funkcija za proporcionalnu zastupljenost

get_props = function(dataSet, heartDiseaseCol, colName) {
  pom = xtabs(as.formula(paste0("~ ", colName, " + ", heartDiseaseCol)), data = dataSet)
  
  res = prop.table(pom, margin = 1)
  res = round(res, 2) * 100
  
  return(res)
}


# Smoking
create_bivariant_bar(health_dataset, "HeartDisease", "Smoking")
get_props(health_dataset, "HeartDisease", "Smoking")

# provera da li starosne neke starosne grupe puse cesce 
data <- health_dataset %>%
  count(AgeCategory, Smoking) %>%  # Prebroj koliko ima pušača i nepušača po starosnim grupama
  group_by(AgeCategory) %>%
  mutate(Percent = n / sum(n) * 100)  # Izračunavanje procenata po starosnoj grupi

# Plotovanje stacked bar chart-a
ggplot(data, aes(x = AgeCategory, y = Percent, fill = Smoking)) +
  geom_bar(stat = "identity") +
  labs(x = "Age category",
       y = "Percent smoking(%)",
       fill = "Smoking") +
  scale_fill_manual(values = c("Yes" = "lightblue", "No" = "pink")) +
  theme_minimal()



#AlcoholDrinking
create_bivariant_bar(health_dataset, "HeartDisease", "AlcoholDrinking")
get_props(health_dataset, "HeartDisease", "AlcoholDrinking")


# Stroke
create_bivariant_bar(health_dataset, "HeartDisease", "Stroke")
get_props(health_dataset, "HeartDisease", "Stroke")


# DiffWalking
create_bivariant_bar(health_dataset, "HeartDisease", "DiffWalking")
get_props(health_dataset, "HeartDisease", "DiffWalking")


#Sex
create_bivariant_bar(health_dataset, "HeartDisease", "Sex")
get_props(health_dataset, "HeartDisease", "Sex")


# AgeCategory
ggplot(health_dataset, aes(x = AgeCategory, fill = HeartDisease)) + 
  geom_bar(position = "dodge") + 
  ylab("HeartDisease") + 
  xlab("AgeCategory") 
  
get_props(health_dataset, "HeartDisease", "AgeCategory")


# Race
ggplot(health_dataset, aes(x = Race, fill = HeartDisease)) + 
  geom_bar(position = "dodge") + 
  ylab("HeartDisease") + 
  xlab("Race") 

get_props(health_dataset, "HeartDisease", "Race")


# Diabetic
ggplot(health_dataset, aes(x = Diabetic, fill = HeartDisease)) + 
  geom_bar(position = "dodge") + 
  ylab("HeartDisease") + 
  xlab("Diabetic") 

get_props(health_dataset, "HeartDisease", "Diabetic")



# PhysicalActivity
create_bivariant_bar(health_dataset, "HeartDisease", "PhysicalActivity")
get_props(health_dataset, "HeartDisease", "PhysicalActivity")


# GenHealth
ggplot(health_dataset, aes(x = GenHealth, fill = HeartDisease)) + 
  geom_bar(position = "dodge") + 
  ylab("HeartDisease") + 
  xlab("GenHealth") 

get_props(health_dataset, "HeartDisease", "GenHealth")


# Asthma
create_bivariant_bar(health_dataset, "HeartDisease", "Asthma")
get_props(health_dataset, "HeartDisease", "Asthma")


# KidneyDisease
create_bivariant_bar(health_dataset, "HeartDisease", "KidneyDisease")
get_props(health_dataset, "HeartDisease", "KidneyDisease")



# SkinCancer
create_bivariant_bar(health_dataset, "HeartDisease", "SkinCancer")
get_props(health_dataset, "HeartDisease", "SkinCancer")





############################################################################################################################################
#   MODELI
############################################################################################################################################

proba = health_dataset
health_dataset = proba

# podela skupa podataka na skup za trening
set.seed(432)
# ukupan broj redova
total_rows = nrow(health_dataset)

# nasumican uzorak indeksa za trening skup
train_indices = sample(1:total_rows, size = 0.7 * total_rows)

# podela na trening i test skup
trainData = health_dataset[train_indices, ]
testData = health_dataset[-train_indices, ]

nrow(trainData)
nrow(testData)


ggplot(trainData)+
  geom_bar(aes(x = HeartDisease, fill = HeartDisease, color = HeartDisease)) +
  labs(title = "Training data", x = "HeartDisease", y = "Count") +
  scale_fill_manual(values = c("skyblue","pink"))+
  scale_color_manual(values = c("blue", "red"))


ggplot(testData)+
  geom_bar(aes(x = HeartDisease, fill = HeartDisease, color = HeartDisease)) +
  labs(title = "Test data", x = "HeartDisease", y = "Count") +
  scale_fill_manual(values = c("skyblue","pink"))+
  scale_color_manual(values = c("blue", "red"))





######################
# Linearna regresija

# Pretvori 'HeartDisease' u binarnu numeričku varijablu
trainData$HeartDisease <- ifelse(trainData$HeartDisease == "Yes", 1, 0)
testData$HeartDisease <- ifelse(testData$HeartDisease == "Yes", 1, 0)

# Ponovo treniraj logistički regresioni model
logistic_model <- glm(HeartDisease ~ ., data = trainData, family = binomial)

# Pogledaj sažetak modela
summary(logistic_model)

# Predikcija na test skupu
predictions <- predict(logistic_model, newdata = testData, type = "response")

# Binarizacija rezultata na osnovu praga od 0.5
predicted_classes <- ifelse(predictions > 0.5, 1, 0)

# Kreiranje konfuzione matrice za evaluaciju modela
confusion_matrix <- confusionMatrix(as.factor(predicted_classes), as.factor(testData$HeartDisease))

# Prikaz rezultata
print(confusion_matrix)

# Vizuelizacija matrice konfuzije
cm_table <- confusion_matrix$table

# Konvertovanje u podatke koje ggplot može koristiti
cm_data <- as.data.frame(cm_table)
colnames(cm_data) <- c("Prediction", "Reference", "Freq")

# Plotovanje pomoću ggplot2
ggplot(data = cm_data, aes(x = Reference, y = Prediction, fill = Freq)) +
  geom_tile() +
  geom_text(aes(label = Freq), color = "white", size = 6) +
  scale_fill_gradient(low = "pink", high = "darkviolet") +
  labs(title = "Confusion Matrix of Linear Regression", x = "Actual", y = "Predicted") +
  theme_minimal()

# acc
accuracy <- confusion_matrix$overall['Accuracy']
cat("Tačnost (Accuracy):", accuracy, "\n")

# sens
sensitivity <- confusion_matrix$byClass['Sensitivity']
cat("Senzitivnost (Sensitivity):", sensitivity, "\n")

# spec
specificity <- confusion_matrix$byClass['Specificity']
cat("Specifičnost (Specificity):", specificity, "\n")


# f1s
f1 <- confusion_matrix$byClass['F1']
cat("F1-score:", f1, "\n")





######################
# xgboost

# Konvertuj dataset u numeričke vrednosti (XGBoost zahteva numeričke podatke)
train_matrix <- model.matrix(HeartDisease ~ . - 1, data = trainData) # -1 da se ukloni intercept
test_matrix <- model.matrix(HeartDisease ~ . - 1, data = testData)

# Ciljna varijabla mora biti u numeričkom formatu
train_labels <- as.numeric(trainData$HeartDisease) - 1  # 0 i 1 format
test_labels <- as.numeric(testData$HeartDisease) - 1

# Kreiraj DMatrix format za XGBoost
dtrain <- xgb.DMatrix(data = train_matrix, label = train_labels)
dtest <- xgb.DMatrix(data = test_matrix, label = test_labels)

# Postavi parametre za XGBoost
params <- list(
  objective = "binary:logistic",  # Binarnu klasifikaciju
  eval_metric = "error",          # Metrika 'error'
  max_depth = 8,                  # Veća dubina stabla
  eta = 0.05,                     # Manji korak učenja
  nthread = 4,                    # Broj jezgara
  alpha = 0.1,                    # L1 regularizacija
  lambda = 1.0                    # L2 regularizacija
)

# Treniraj XGBoost model
xgb_model <- xgb.train(params = params,
                       data = dtrain,
                       nrounds = 100,   # Broj iteracija (stabala)
                       watchlist = list(train = dtrain),
                       verbose = 1)

# Predikcija na test setu
predictions_xgb <- predict(xgb_model, newdata = dtest)

# Konvertuj predikcije u binarne klase
predicted_classes_xgb <- ifelse(predictions_xgb > 0.5, 1, 0)

# Provera nivoa predikovanih i stvarnih vrednosti
actual_classes_xgb <- as.numeric(testData$HeartDisease) - 1

# Napravi matricu konfuzije
confusion_matrix_xgb <- confusionMatrix(factor(predicted_classes_xgb), 
                                        factor(actual_classes_xgb))

# Prikaz svih metrika
print(confusion_matrix_xgb)

# Vizuelizacija matrice konfuzije
cm_table <- confusion_matrix_xgb$table

# Konvertovanje u podatke koje ggplot može koristiti
cm_data <- as.data.frame(cm_table)
colnames(cm_data) <- c("Prediction", "Reference", "Freq")

# Plotovanje pomoću ggplot2
ggplot(data = cm_data, aes(x = Reference, y = Prediction, fill = Freq)) +
  geom_tile() +
  geom_text(aes(label = Freq), color = "white", size = 6) +
  scale_fill_gradient(low = "pink", high = "darkviolet") +
  labs(title = "Confusion Matrix of xgboost", x = "Actual", y = "Predicted") +
  theme_minimal()

# acc
accuracy <- confusion_matrix_xgb$overall['Accuracy']
cat("Tačnost (Accuracy):", accuracy, "\n")

# sens
sensitivity <- confusion_matrix_xgb$byClass['Sensitivity']
cat("Senzitivnost (Sensitivity):", sensitivity, "\n")

# spec
specificity <- confusion_matrix_xgb$byClass['Specificity']
cat("Specifičnost (Specificity):", specificity, "\n")


# f1s
f1 <- confusion_matrix_xgb$byClass['F1']
cat("F1-score:", f1, "\n")






